<?php

declare(strict_types=1);

namespace Drupal\activity_stream\Plugin\rest\resource;

use Drupal\Core\KeyValueStore\KeyValueFactoryInterface;
use Drupal\Core\KeyValueStore\KeyValueStoreInterface;
use Drupal\rest\ModifiedResourceResponse;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\HttpKernel\Exception\UnprocessableEntityHttpException;
use Symfony\Component\Routing\Route;
use Drupal\activity_stream\Helper;
use Drupal\message\MessageInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Pager\PagerManagerInterface;

/**
 * Represents Stream records as resources.
 *
 * @RestResource (
 *   id = "activity_stream_stream",
 *   label = @Translation("Stream"),
 *   uri_paths = {
 *     "canonical" = "/api/activity-stream-stream/{destination}/{user}",
 *   }
 * )
 *
 * @DCG
 * The plugin exposes key-value records as REST resources. In order to enable it
 * import the resource configuration into active configuration storage. An
 * example of such configuration can be located in the following file:
 * core/modules/rest/config/optional/rest.resource.entity.node.yml.
 * Alternatively, you can enable it through admin interface provider by REST UI
 * module.
 * @see https://www.drupal.org/project/restui
 *
 * @DCG
 * Notice that this plugin does not provide any validation for the data.
 * Consider creating custom normalizer to validate and normalize the incoming
 * data. It can be enabled in the plugin definition as follows.
 * @code
 *   serialization_class = "Drupal\foo\MyDataStructure",
 * @endcode
 *
 * @DCG
 * For entities, it is recommended to use REST resource plugin provided by
 * Drupal core.
 * @see \Drupal\rest\Plugin\rest\resource\EntityResource
 */
final class StreamResource extends ResourceBase {

  /**
   * The key-value storage.
   */
  private readonly KeyValueStoreInterface $storage;

  /**
   * The Helper service.
   */
  private readonly Helper $helperService;  

  /**
   * The Page manager.
   */
  private readonly PagerManagerInterface $pagerManager;    

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    array $serializer_formats,
    LoggerInterface $logger,
    KeyValueFactoryInterface $keyValueFactory,
    Helper $helper_service,
    PagerManagerInterface $pager_manager
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger, $helper_service, $pager_manager);
    $this->storage = $keyValueFactory->get('activity_stream_stream');
    $this->helperService  = $helper_service;
    $this->pagerManager = $pager_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new self(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('rest'),
      $container->get('keyvalue'),
      $container->get('activity_stream.helper'),
      $container->get('pager.manager'),
    );
  }

  /**
   * Responds to GET requests.
   */
  public function get($destination, $user): ModifiedResourceResponse {

    $data = [];

    $activity_date_from = \Drupal::request()->query->get('activity_date_from');
    $activity_date_to = \Drupal::request()->query->get('activity_date_to');
    $activity_entity_type = \Drupal::request()->query->get('activity_entity_type');
    $activity_entity_bundle = \Drupal::request()->query->get('activity_entity_bundle');
    $current_page = \Drupal::request()->query->get('current_page');

    try {
      // Validate parameters
      if (!$this->helperService->validateDestination($destination)) {
        throw new BadRequestHttpException('The destination is not a valid.');
      } 
      if (!$this->helperService->isValidUserUUID($user)) {
        throw new BadRequestHttpException('The user is not a valid user uuid.');
      }
      if (
          isset($activity_date_from) && 
          !$this->helperService->isValidTimestamp($activity_date_from)
      ) {
        throw new BadRequestHttpException('The activity_date_from is not a valid timestamp.');
      }
      if (
        isset($activity_date_to) && 
        !$this->helperService->isValidTimestamp($activity_date_to)
      ) {
        throw new BadRequestHttpException('The activity_date_to is not a valid timestamp.');
      }
      if (
        isset($activity_entity_type) &&
        !$this->helperService->isValidEntityType($activity_entity_type)
      ) {
        throw new BadRequestHttpException('The activity_entity_type is not a valid entity type.');
      }
      if (
        isset($activity_entity_bundle) &&
        !$this->helperService->isValidBundle($activity_entity_bundle)
      ) {
        throw new BadRequestHttpException('The activity_entity_bundle is not a valid bundle.');
      }

      $query = \Drupal::entityQuery('activity_stream_activity')
        ->condition('field_activity_destinations', $destination, 'IN');
        //->condition('field_activity_date.value', $activity_date_from, '>=')
        //->condition('field_activity_date.value', $activity_date_to, '<=');

      if (isset($activity_entity_type) && isset($activity_entity_bundle)) {  
        if ($activity_entity_type !== $activity_entity_bundle) {
          $query->condition("field_activity_entity.entity:$activity_entity_type.bundle", $activity_entity_bundle, '=');
        }
        else {
          $query->condition('field_activity_entity.target_type', $activity_entity_type, '=');
        }
      }

      $current_page_now = $current_page * 50;

      $query->accessCheck(TRUE);
      $query->range($current_page_now,50);
      //$query->pager();
      $results = $query->execute();  
      
      if (!empty($results)) {

        $entities = \Drupal::entityTypeManager()->getStorage('activity_stream_activity')->loadMultiple($results);

        foreach ($entities as $entity) {
          // Get the activiy entity
          $activity_entity = $entity->field_activity_entity->entity;
          if ($activity_entity instanceof EntityInterface) {
            $data[]['activity_entity']['data'][] = $activity_entity;
          }
          // Get the message entity
          $activity_message = $entity->field_activity_message->entity;
          if ($activity_message instanceof MessageInterface) {
            $data[]['activity_message']['data'][] = $activity_message->getText();
          }
        }

        $pager = $this->pagerManager->createPager(count($results), 50);

        if ($pager instanceof \Drupal\Core\Pager\Pager) {

          if ($pager->getTotalItems() > 0) {
            $next_page = $current_page +  1;
          }
          else {
            $next_page = NULL;
          }

          if ($current_page > 0) {
            $prev_page = $current_page - 1;
          }
          else {
            $prev_page = NULL;
          }

          $pager_data = [
            'total_items' => $pager->getTotalItems(),
            'total_pages' => $pager->getTotalPages(),
            'current_page' => $current_page,
            'next_page' => $next_page,
            'limit' => $pager->getLimit(),
          ]; 
          $data[]['pager'][] = $pager_data; 
        } 

        $response = new ModifiedResourceResponse($data, 201);

      }
      else {
        throw new UnprocessableEntityHttpException('No data found.');
      }
    }
    catch (BadRequestHttpException $exception) {
      $response = new ModifiedResourceResponse([
       'error' => $exception->getMessage() 
      ]);
    }
    catch (UnprocessableEntityHttpException $exception) {
      $response = new ModifiedResourceResponse([
        'error' => $exception->getMessage() 
       ]);
    }    
    catch (\Exception $exception) {
      $response = new ModifiedResourceResponse([
        'error' => $exception->getMessage() 
       ]);
    }

    return $response;  

  }


}
